#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/openssh/Regression/bz915316-ssh-segfaults-when-trying-to-authenticate-ldap
#   Description: Test for BZ#915316 (ssh segfaults when trying to authenticate ldap)
#   Author: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/bin/rhts-environment.sh || exit 1
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="openssh"

PACKAGES=( "pam_ldap"         \
           "nss-pam-ldapd"    \
           "openssh"          \
           "expect"           \
           "authconfig"       \
           "openldap"         \
           "openldap-servers" )

CONFIG_FILE="pam_ldap"
LDAP_SERVICE="slapd"
BASE64_OPTION="-d"                    

BDN='cn=Manager,dc=my-domain,dc=com'
PWD='x'


function get_ssh_fingerprint {
expect <<EOF
set timeout 10
spawn ssh ldapuser@my-domain.com
expect {
	"*(yes/no)?*" { send "yes\r"; exit 0}
	timeout { puts "timeout"; exit 0}
}
    
exit 0
EOF
}

function do_ssh {
expect <<EOF
set timeout 20
spawn ssh ldapuser@my-domain.com
expect {
	"ldapuser@my-domain.com's password:" { puts "x" ; send "x\r" }
	timeout { puts "timeout" ; exit 2 } 
	eof { puts "eof" ; exit 3 } 
}
expect {
	-re ".*$" { puts "get command line" } 
	timeout { puts "timeout" ; exit 4 } 
	eof { puts "eof" ; exit 5 } 
}
expect "ldapuser@my-domain.com's password:" { puts "wrong password"; exit 6 }
send "whoami\r"
expect {
	"ldapuser" { exit 0 }
	timeout { puts "timeout" ; exit 7 } 
	eof { puts "eof" ; exit 8 } 
}
    
exit 1
EOF
}

function data_ldif {
cat  >data.ldif <<EOF
dn: dc=my-domain,dc=com
dc: my-domain
objectClass: top
objectClass: domain

dn: ou=people,dc=my-domain,dc=com
ou: people
objectClass: top
objectClass: organizationalUnit

dn: uid=ldapuser,ou=people,dc=my-domain,dc=com
uid: ldapuser
cn: ldapuser
objectClass: account
objectClass: posixAccount
objectClass: top
userPassword: {SSHA}A41wdK4LTqBbyqqeWxHARusxQClMYwTy
loginShell: /bin/bash
uidNumber: 1001
gidNumber: 1000
homeDirectory: /home/ldapuser
gecos: ldapuser

dn: uid=binddn,ou=people,dc=my-domain,dc=com
uid: binddn
cn: binddn
objectClass: account
objectClass: posixAccount
objectClass: top
userPassword: {SSHA}A41wdK4LTqBbyqqeWxHARusxQClMYwTy
loginShell: /bin/bash
uidNumber: 1002
gidNumber: 1000
homeDirectory: /home/binddn
gecos: binddn
EOF
return $?
}

function slapd_conf {
cat  >/etc/openldap/slapd.conf <<EOF
include         /etc/openldap/schema/corba.schema
include         /etc/openldap/schema/core.schema
include         /etc/openldap/schema/cosine.schema
include         /etc/openldap/schema/duaconf.schema
include         /etc/openldap/schema/dyngroup.schema
include         /etc/openldap/schema/inetorgperson.schema
include         /etc/openldap/schema/java.schema
include         /etc/openldap/schema/misc.schema
include         /etc/openldap/schema/nis.schema
include         /etc/openldap/schema/openldap.schema
include         /etc/openldap/schema/ppolicy.schema
include         /etc/openldap/schema/collective.schema

allow bind_v2

loglevel 1 256

pidfile         /var/run/openldap/slapd.pid
argsfile        /var/run/openldap/slapd.args

TLSCipherSuite        HIGH:MEDIUM:+SSLv2
TLSCACertificateFile  /etc/openldap/cacerts/cacert.pem
TLSCertificateFile    /etc/openldap/cacerts/servercrt.pem
TLSCertificateKeyFile /etc/openldap/cacerts/serverkey.pem

database        bdb
suffix          "dc=my-domain,dc=com"
rootdn          "cn=Manager,dc=my-domain,dc=com"

# Password is 'x'.
rootpw          {SSHA}GPhzu7pTYP4I+nGeujpBkODiPxX0v8n8

directory       /var/lib/ldap/

index objectClass                       eq,pres
index ou,cn,mail,surname,givenname      eq,pres,sub
index uidNumber,gidNumber,loginShell    eq,pres
index uid,memberUid                     eq,pres,sub
index nisMapName,nisMapEntry            eq,pres,sub

access to attrs=shadowLastChange,userPassword
      by self write
      by * auth

access to *
      by * read
EOF
return $?
}

rlJournalStart
    rlPhaseStartSetup
	for P in ${PACKAGES[@]}; do
		rlCheckRpm $P || rlDie "Package $P is missing"
	done

	rlServiceStop "nscd"
	rlServiceStop "nslcd"
	rlServiceStop $LDAP_SERVICE

	rlFileBackup "/etc/$CONFIG_FILE.conf"       
	rlFileBackup --clean "/etc/openldap/slapd.conf"
	rlFileBackup "/etc/openldap/ldap.conf"      
	rlFileBackup --clean "/etc/openldap/slapd.d" && rm -rf /etc/openldap/slapd.d/*
	rlFileBackup --clean "/var/lib/ldap/" && rm -rf /var/lib/ldap/*
	rlFileBackup --clean "/var/run/openldap/" && rm -rf /var/run/openldap/*
	rlFileBackup "/etc/hosts" 
	rlFileBackup "/etc/nsswitch.conf"             

	rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
	                
	rlRun "slapd_conf" 0
	slaptest -f /etc/openldap/slapd.conf -F /etc/openldap/slapd.d/
	rlRun "chown -R ldap:ldap /etc/openldap/slapd.d/" 0

	# Loading testing data into OpenLDAP servers.
	rlRun "data_ldif" 0
	rlRun "slapadd -l data.ldif" 0

	rlRun "chown -R ldap:ldap /var/lib/ldap" 0

	#Prepare password for rootbinddn option of pam_ldap
#	rlRun "echo 'x' > /etc/$CONFIG_FILE.secret" 0
#	rlRun "chmod 0600 /etc/$CONFIG_FILE.secret" 0

	rlRun "restorecon -Rv /etc/openldap"    

	rlRun "service $LDAP_SERVICE start && sleep 10" 0

	#BZ#755975
	# maybe problem with return value 1 of authconfig --savebackup, funcionality ok, but returns 1 instead of 0
	rlRun "authconfig --savebackup BZ915316" 0      
    rlPhaseEnd

    rlPhaseStartTest
	rlRun "authconfig --enableldap --enableforcelegacy --enablemkhomedir --disablecache --enableldapauth --updateall \
	  --ldapbasedn dc=my-domain,dc=com --ldapserver ldap://my-domain.com" 0 \
	  "Configuring authentication mechanisms" 

	rlRun "get_ssh_fingerprint" 0
	rlRun "do_ssh" 0

	rlRun "echo 'binddn cn=Manager,dc=my-domain,dc=com' >> /etc/nslcd.conf" 0
	rlRun "echo 'bindpw x' >> /etc/nslcd.conf" 0
	rlRun "service nslcd restart & sleep 5" 0

	rlRun "do_ssh" 0
    rlPhaseEnd

    rlPhaseStartCleanup
	rlRun "service $LDAP_SERVICE stop" 0
	rlRun "service nslcd stop" 0
	rlRun "authconfig --restorebackup BZ915316"

	rm -rf /home/ldapuser

	rlFileRestore

	rlServiceRestore "nscd"
	rlServiceRestore "nslcd"
	rlServiceRestore $LDAP_SERVICE
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
